<?php

namespace Drupal\Tests\jsonapi\Unit\Normalizer;

use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\jsonapi\ResourceType\ResourceType;
use Drupal\jsonapi\ResourceType\ResourceTypeRepository;
use Drupal\jsonapi\Normalizer\ConfigEntityNormalizer;
use Drupal\jsonapi\LinkManager\LinkManager;
use Drupal\jsonapi\Normalizer\ScalarNormalizer;
use Drupal\Tests\UnitTestCase;
use Prophecy\Argument;
use Symfony\Component\Serializer\Serializer;

/**
 * @coversDefaultClass \Drupal\jsonapi\Normalizer\ConfigEntityNormalizer
 * @group jsonapi
 */
class ConfigEntityNormalizerTest extends UnitTestCase {

  /**
   * The normalizer under test.
   *
   * @var \Drupal\jsonapi\Normalizer\ConfigEntityNormalizer
   */
  protected $normalizer;

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    $link_manager = $this->prophesize(LinkManager::class);

    $resource_type_repository = $this->prophesize(ResourceTypeRepository::class);
    $resource_type_repository->get(Argument::type('string'), Argument::type('string'))
      ->willReturn(new ResourceType('dolor', 'sid', NULL));

    $this->normalizer = new ConfigEntityNormalizer(
      $link_manager->reveal(),
      $resource_type_repository->reveal(),
      $this->prophesize(EntityTypeManagerInterface::class)->reveal()
    );

    $normalizers = [new ScalarNormalizer()];
    $serializer = new Serializer($normalizers, []);
    $this->normalizer->setSerializer($serializer);
  }

  /**
   * @covers ::normalize
   * @dataProvider normalizeProvider
   */
  public function testNormalize($input, $expected) {
    $entity = $this->prophesize(ConfigEntityInterface::class);
    $entity->toArray()->willReturn(['amet' => $input]);
    $entity->getCacheContexts()->willReturn([]);
    $entity->getCacheTags()->willReturn([]);
    $entity->getCacheMaxAge()->willReturn(-1);
    $entity->getEntityTypeId()->willReturn('');
    $entity->bundle()->willReturn('');
    $normalized = $this->normalizer->normalize($entity->reveal(), 'api_json', []);
    $first = $normalized->getValues();
    $first = reset($first);
    $this->assertSame($expected, $first->rasterizeValue());
  }

  /**
   * Data provider for the normalize test.
   *
   * @return array
   *   The data for the test method.
   */
  public function normalizeProvider() {
    return [
      ['lorem', 'lorem'],
      [
        ['ipsum' => 'dolor', 'ra' => 'foo'],
        ['ipsum' => 'dolor', 'ra' => 'foo'],
      ],
      [['ipsum' => 'dolor'], 'dolor'],
      [
        ['lorem' => ['ipsum' => ['dolor' => 'sid', 'amet' => 'ra']]],
        ['ipsum' => ['dolor' => 'sid', 'amet' => 'ra']],
      ],
    ];
  }

}
